/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.resources;

import ca.uhn.fhir.context.FhirContext;
import ca.uhn.fhir.parser.IParser;
import ca.uhn.fhir.rest.client.api.IGenericClient;
import ca.uhn.fhir.rest.client.interceptor.BasicAuthInterceptor;
import ca.uhn.fhir.rest.gclient.ReferenceClientParam;
import jakarta.annotation.Nonnull;
import org.hl7.fhir.r4.model.Bundle;
import org.hl7.fhir.r4.model.Enumerations;
import org.hl7.fhir.r4.model.Patient;
import org.hl7.fhir.r4.model.Reference;
import org.hl7.fhir.r4.model.SearchParameter;
import org.hl7.fhir.r4.model.StringType;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;

/**
 * This class demonstrates a HAPI FHIR client interacting with a
 * server that has implemented the custom resources in this package.
 * <p>
 * This class is not needed in order to use the custom resource classes,
 * it is just a demonstration.
 */
public class ExampleClient {
	private static final Writer OUT = new PrintWriter(System.out, true, StandardCharsets.UTF_8);

	public static void main(String[] theArgs) throws IOException {

		// Create a FhirContext and register the custom types

		FhirContext ctx = FhirContext.forR4();
		ctx.registerCustomType(Meal.class);
		ctx.registerCustomType(MealIngredient.class);

		// Create a client to interact with your server

		IGenericClient client = ctx.newRestfulGenericClient("http://localhost:8000");
		client.registerInterceptor(new BasicAuthInterceptor("admin", "password"));

		// Create Some Search Parameters

		SearchParameter mealChefSearchParameter = createMealChefSearchParameter();
		client.update().resource(mealChefSearchParameter).execute();

		SearchParameter mealNameSearchParameter = createMealNameSearchParameter();
		client.update().resource(mealNameSearchParameter).execute();

		// Create some sample resources

		Patient patient = createPatientResource();
		client.update().resource(patient).execute();

		Meal meal1 = createMealResource1();
		client.update().resource(meal1).execute();

		Meal meal2 = createMealResource2();
		client.update().resource(meal2).execute();

		// Perform a Search for Meals belonging to the Patient

		Bundle results = client.search()
				.forResource(Meal.class)
				.where(new ReferenceClientParam("chef").hasId("Patient/P"))
				.returnBundle(Bundle.class)
				.execute();
		IParser parser = ctx.newJsonParser().setPrettyPrint(true);
		OUT.write("Response:\n" + parser.encodeResourceToString(results));
		OUT.flush();
	}

	public static SearchParameter createMealChefSearchParameter() {
		SearchParameter sp = new SearchParameter();
		sp.setId("SearchParameter/meal-chef");
		sp.setUrl("http://example.org/SearchParameter/meal-chef");
		sp.addBase("Meal");
		sp.setCode("chef");
		sp.setType(Enumerations.SearchParamType.REFERENCE);
		sp.setStatus(Enumerations.PublicationStatus.ACTIVE);
		sp.setExpression("Meal.chef");
		sp.addTarget("Patient");
		return sp;
	}

	@Nonnull
	public static SearchParameter createMealNameSearchParameter() {
		SearchParameter sp = new SearchParameter();
		sp.setId("SearchParameter/meal-name");
		sp.setUrl("http://example.org/SearchParameter/meal-name");
		sp.addBase("Meal");
		sp.setCode("name");
		sp.setType(Enumerations.SearchParamType.STRING);
		sp.setStatus(Enumerations.PublicationStatus.ACTIVE);
		sp.setExpression("Meal.name");
		return sp;
	}

	@Nonnull
	public static Meal createMealResource1() {
		Meal meal = new Meal();
		meal.setId("Meal/meal1");
		meal.setName(new StringType("Pizza"));
		meal.setChef(new Reference("Patient/P"));
		return meal;
	}

	@Nonnull
	public static Meal createMealResource2() {
		Meal meal = new Meal();
		meal.setId("Meal/meal2");
		meal.setName(new StringType("Burgers"));
		meal.setChef(new Reference("Patient/P"));
		return meal;
	}

	@Nonnull
	public static Patient createPatientResource() {
		Patient p = new Patient();
		p.setId("Patient/P");
		p.addName().setFamily("Simpson").addGiven("Homer");
		return p;
	}
}
